unit MyMath;

{$mode objfpc}{$H+}
{$modeswitch advancedrecords}

interface

uses
  Classes, SysUtils, Types, Math;

type

  TPointf = record
    x, y: single;
  end;

  { TSchwinge }

  TSchwinge = record
    Pleuel_W, Schwinge_W: single;
    RadExzenter: TPointF;
    procedure SetParam(Rad_W: single; Rad_R, Schwinge_x, Schwinge_y, Pleuel_L, Schwinge_L: single);
  end;

  { TTrigo }

  TTrigo = record
    Grad: boolean;
    function a_b_c_To_Alpha(a, b, c: single): single;
    function a_b_c_To_Beta(a, b, c: single): single;
    function a_b_c_To_Gama(a, b, c: single): single;

    // Dreieck rechtwinklig

    function Alpha_c_To_xy(angle, len: single): TPointf;

    function a_c_To_Alpha(a, c: single): single;
    function a_c_To_Beta(a, c: single): single;
    function b_c_To_Alpha(b, c: single): single;
    function b_c_To_Beta(b, c: single): single;

    function a_b_To_Alpha(a, b: single): single;
  end;

var
  Trigo: TTrigo;


implementation

{ TTrigo }

function TTrigo.a_b_c_To_Alpha(a, b, c: single): single;
begin
  Result := arccos((-sqr(a) + sqr(b) + sqr(c)) / (2 * b * c));
  if Grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.a_b_c_To_Beta(a, b, c: single): single;
begin
  Result := arccos((-sqr(b) + sqr(a) + sqr(c)) / (2 * a * c));
  if Grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.a_b_c_To_Gama(a, b, c: single): single;
begin
  Result := arccos((-sqr(c) + sqr(a) + sqr(b)) / (2 * a * b));
  if Grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.Alpha_c_To_xy(angle, len: single): TPointf;
begin
  if Grad then begin
    angle := angle / 180.0 * Pi;
  end;
  Result.x := cos(angle) * len;
  Result.y := sin(angle) * len;
end;

function TTrigo.a_c_To_Alpha(a, c: single): single;
begin
  Result := arcsin(a / c);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.a_c_To_Beta(a, c: single): single;
begin
  Result := arccos(a / c);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.b_c_To_Alpha(b, c: single): single;
begin
  Result := arccos(b / c);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.b_c_To_Beta(b, c: single): single;
begin
  Result := arcsin(b / c);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.a_b_To_Alpha(a, b: single): single;
begin
  Result := arctan(a / b);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;



{ TSchwinge }

procedure TSchwinge.SetParam(Rad_W: single; Rad_R, Schwinge_x, Schwinge_y, Pleuel_L, Schwinge_L: single);
var
  c,           // Ist der Abstand zwischen dem Radexzenter und dem Zentrum der Schwinge
  w: single;   // Ist der Winkel von C
begin
  RadExzenter := Trigo.Alpha_c_To_xy(Rad_W, Rad_R);

  c := sqrt(sqr(RadExzenter.y - Schwinge_y) + sqr(Schwinge_x - RadExzenter.x));
  w := arcsin((-RadExzenter.y + Schwinge_y) / c) / Pi * 180.0;

  Pleuel_W := Trigo.a_b_c_To_Alpha(Schwinge_L, Pleuel_L, c) + w;
  Schwinge_W := 180.0 - Trigo.a_b_c_To_Beta(Schwinge_L, Pleuel_L, c) + w;
end;


end.
